import os
from abc import ABC, abstractmethod
from typing import Union, Dict, Type, Text, Optional

from django.forms import Form
from django.http import QueryDict
from django.urls import reverse

from bi.lib import get_class_by_path
from bi.models.object import BaseObject


class BaseDashboard(BaseObject, ABC):
    """Base abstract class for all dashboards.

    Attributes:
        params: Dashboard parameters.
    """

    params: Dict

    def __init__(self, params: Dict):
        """Inits dashboard.

        Args:
            params: Dashboard parameters.
        """
        super().__init__()
        self.params = params

    @property
    def id(self) -> Text:
        """Returns id of dashboard.

        Returns:
            A string with id of dashboard.
        """
        return str(self.__module__).split('.')[-1]

    @property
    def icon(self) -> Text:
        """Returns icon of dashboard.

        Returns:
            A string with icon of dashboard.
        """
        return "fa fa-pie-chart"

    @property
    def form_class(self) -> Union[Type[Form], None]:
        """Returns dashboards's form class.
        """
        return None

    @property
    def form_defaults(self) -> Dict:
        """Returns dashboards's default form inputs values.
        """
        return {}

    @property
    @abstractmethod
    def title(self) -> Text:
        """Returns title of dashboard.

        Returns:
            A string with title of dashboard.
        """
        pass

    @property
    def template(self) -> Text:
        """Returns path to template of dashboard.

        Returns:
            A string with path to template of dashboard.
        """
        if self.get_parent_dashboard_id():
            return os.path.join('dashboards', self.get_parent_dashboard_id(), self.id) + '.html'
        else:
            return os.path.join('dashboards', self.id) + '.html'

    @classmethod
    def get_parent_dashboard_id(cls) -> Union[Text, None]:
        """Returns id of dashboard's parent.

        Returns:
            ID of dashboard's parent.
        """
        module_splitted = cls.__module__.split('.')
        if module_splitted[-2] == 'dashboards':
            return None
        else:
            return module_splitted[-2]

    @classmethod
    def has_parent_dashboard(cls) -> bool:
        """Returns True if dashboard has parent dashboard.

        Returns:
            True of False.
        """
        module_splitted = cls.__module__.split('.')
        return module_splitted[-2] != 'dashboards'

    @classmethod
    def get_parent_dashboard_url(cls) -> Union[Text, None]:
        """Returns url of parent dashboard.

        Returns:
            URL of dashboard's parent or None.
        """
        if cls.has_parent_dashboard():
            return reverse('bi:dashboard-detail', args=[cls.get_parent_dashboard_id()])
        else:
            return None

    @classmethod
    def get_parent_dashboard_class(cls) -> Union[Text, None]:
        """Returns class of dashboard's parent.

        Returns:
            Class of dashboard's parent.
        """
        module_splitted = cls.__module__.split('.')
        return get_class_by_path(os.path.join(*module_splitted[:-1]) + '.py', 'Dashboard')

    @classmethod
    def get_parent_dashboard(cls):
        """Returns class of dashboard's parent.

        Returns:
            Class of dashboard's parent.
        """
        module_splitted = cls.__module__.split('.')
        return get_class_by_path(os.path.join(*module_splitted[:-1]) + '.py', 'Dashboard')({})

    def get_form(self) -> Optional[Type[Form]]:
        """Returns form instance.

        Returns:
            Form instance.
        """
        if self.has_form():
            params = QueryDict(mutable=True)
            params.update(self.form_defaults)
            params.update(self.params)

            form = self.form_class(params)
            form.is_valid()

            return form

    def has_form(self) -> bool:
        """Returns True if dashboard has form.

        Returns:
            True or False.
        """
        return self.form_class is not None



from django.contrib.auth.decorators import login_required
from django.core.cache import cache
from django.http import Http404
from django.http import JsonResponse
from django.shortcuts import render

from bi.lib import get_entity_by_path, get_reports_list


@login_required()
def index(request):
    dashboard = get_entity_by_path('dashboards/{}.py'.format('home'), 'Dashboard', request.GET)
    if not dashboard:
        raise Http404()

    return render(request, 'bi/dashboard_detail.html', {'dashboard': dashboard})


@login_required()
def report_list(request):
    reports = get_reports_list()

    context = {'report_list': reports}

    return render(request, 'bi/report_list.html', context)


@login_required()
def report_detail(request, report_path):
    report = get_entity_by_path('reports/{}.py'.format(report_path), 'Report', request.GET)
    if not report:
        raise Http404()

    return render(request, 'bi/report_detail.html', {'report': report})


@login_required()
def report_detail_raw(request, report_path):
    report = get_entity_by_path('reports/{}.py'.format(report_path), 'Report', request.GET)
    if not report:
        raise Http404()

    return report.get_data()


@login_required()
def dashboard_detail(request, dashboard_id):
    dashboard = get_entity_by_path('dashboards/{}.py'.format(dashboard_id), 'Dashboard', request.GET)
    if not dashboard:
        raise Http404()

    return render(request, 'bi/dashboard_detail.html', {'dashboard': dashboard})


@login_required()
def dashboard_detail_nested(request, dashboard_id, dashboard_parent_id):
    dashboard = get_entity_by_path(
        'dashboards/{}/{}.py'.format(dashboard_parent_id, dashboard_id), 'Dashboard', request.GET)
    if not dashboard:
        raise Http404()

    return render(request, 'bi/dashboard_detail.html', {'dashboard': dashboard})


@login_required()
def flush_cache(request):
    cache.clear()
    return JsonResponse({'Result': 'Ok'})



from uuid import uuid4

from django.template import Library, Node, TemplateSyntaxError

register = Library()


class UUIDNode(Node):
    """
    Генерация UUID.
    """

    def __init__(self, var_name):
        self.var_name = var_name

    def render(self, context):
        context[self.var_name] = str(uuid4())
        return ''


def do_uuid(parser, token):
    """
    Генерация UUID

    Использование:
        {% uuid var_name %}
        var_name будет содержать сгенерированный UUID
    """
    try:
        tag_name, var_name = token.split_contents()
    except ValueError:
        raise TemplateSyntaxError("%r tag requires exactly one argument" % token.contents.split()[0])
    return UUIDNode(var_name)


do_uuid = register.tag('uuid', do_uuid)



import hashlib
import urllib.parse

from django import template
from django.utils.safestring import mark_safe

register = template.Library()


@register.filter
def gravatar_url(email, size=40):
    """Returns url of gravatar.

    Usage example:
        {{ email|gravatar_url:150 }}

    Returns:
        A string with url of gravatar by e-mail.
    """
    return "https://www.gravatar.com/avatar/%s?%s" % (
        hashlib.md5(email.lower().encode('utf-8')).hexdigest(), urllib.parse.urlencode({'s': str(size)}))


@register.filter
def gravatar(email, size=40):
    """Returns image html tag with gravatar.

    Usage example:
        {{ email|gravatar:150 }}

    Returns:
        A string with html tag of gravatar by e-mail.
    """
    url = gravatar_url(email, size)
    return mark_safe('<img src="%s" height="%d" width="%d">' % (url, size, size))
